local AceLocale = LibStub ("AceLocale-3.0")
local Loc = AceLocale:GetLocale ("Details_DeathGraphs")

--> create the plugin object
local DeathGraphs = _detalhes:NewPluginObject ("Details_DeathGraphs", DETAILSPLUGIN_ALWAYSENABLED)
tinsert (UISpecialFrames, "Details_DeathGraphs")
--> main frame (shortcut)
local DeathGraphsFrame = DeathGraphs.Frame

local debugmode = true

local function CreatePluginFrames()

	--> basic tables
	DeathGraphs.current_survival_table = {}
	local _combat_object
	
	--> on event stuff
	function DeathGraphs:OnDetailsEvent (event, ...)
		if (event == "HIDE") then --> plugin hidded, disabled
			self.open = false
		
		elseif (event == "SHOW") then --> plugin hidded, disabled
			self.open = true
			
		elseif (event == "COMBAT_PLAYER_ENTER") then --> combat started
			DeathGraphs:ScheduleTimer ("ResetSurvivalTable", 1)
			_combat_object = select (1, ...)
		
		elseif (event == "COMBAT_PLAYER_LEAVE") then --> combat ended
			DeathGraphs:CloseSurvivalTable()
			DeathGraphs:WasBossEncounter()
			
		elseif (event == "DETAILS_DATA_RESET") then
			DeathGraphs:ResetData()
			
		end
	end

	function DeathGraphs:WasBossEncounter()
		if (debugmode or (_combat_object and _combat_object.is_boss)) then
			DeathGraphs:ShowIcon()
		else
			DeathGraphs:ResetData()
			DeathGraphs:HideIcon()
		end
	end
	
	function DeathGraphs:ResetData()
		table.wipe (DeathGraphs.players)
		table.wipe (DeathGraphs.survival)
		table.wipe (DeathGraphs.battleress)
		table.wipe (DeathGraphs.current_survival_table)
		
		DeathGraphs:HideIcon()
		--> call function to reset layout
	end

	--> reset survival table in the beginning of the combat
	function DeathGraphs:ResetSurvivalTable()
	
		--> wipe first
		table.wipe (DeathGraphs.current_survival_table)
		
		--> build the table
		if (IsInRaid()) then
			for i = 1, GetNumGroupMembers(), 1 do
				local name = GetUnitName ("raid"..i, true)
				DeathGraphs.current_survival_table [name] = {alive = true, time = 0, alivesince = time()}
			end
		elseif (IsInGroup()) then
			for i = 1, GetNumGroupMembers()-1, 1 do
				local name = GetUnitName ("raid"..i, true)
				DeathGraphs.current_survival_table [name] = {alive = true, time = 0, alivesince = time()}
			end
		else
			if (debugmode) then
				local name = GetUnitName ("player", true)
				DeathGraphs.current_survival_table [name] = {alive = true, time = 0, alivesince = time()}
				print (time())
			end
		end
	end
	
	--> close and add survival to all player at the and of the combat
	function DeathGraphs:CloseSurvivalTable()
		for name, survival in pairs (DeathGraphs.current_survival_table) do 
			if (survival.alive) then
				--close table
				survival.time = survival.time + (time() - survival.alivesince)
				--add the time
				local playersurvival = DeathGraphs:GetPlayerTable (name, "survival")
				playersurvival.time = playersurvival.time + survival.time
			end
		end
	end
	
	function DeathGraphs:CreatePlayerTable (name, type)
		if (type == "death") then
			DeathGraphs.players [name] = {}
			return DeathGraphs.players [name]
		
		elseif (type == "survival") then
			DeathGraphs.survival [name] = {time = 0}
			return DeathGraphs.survival [name]
			
		elseif (type == "battleress") then
			DeathGraphs.battleress [name] = {}
			return DeathGraphs.battleress [name]
			
		end
	end
	
	function DeathGraphs:GetPlayerTable (name, type)
	
		if (type == "death") then
			local t = DeathGraphs.players [name]
			
			if (not t) then
				t = DeathGraphs:CreatePlayerTable (name, "death")
			end
			
			return t
		
		elseif (type == "survival") then
			local t = DeathGraphs.survival [name]
			
			if (not t) then
				t = DeathGraphs:CreatePlayerTable (name, "survival")
			end
			
			return t

		elseif (type == "battleress") then
			local t = DeathGraphs.battleress [name]
			
			if (not t) then
				t = DeathGraphs:CreatePlayerTable (name, "battleress")
			end
			
			return t
		
		end
	end
	
	function DeathGraphs:AddDeath (serial, name, death_table, last_cooldown, time_of_death, max_health, time)
		
		if (DeathGraphs.in_combat) then
			
			--> death stuff
			local playertable = DeathGraphs:GetPlayerTable (name, "death")
			
			local bossname = ""
			if (_combat_object.is_boss and _combat_object.is_boss.name) then
				bossname = _combat_object.is_boss.name
			end
			
			local _, class = UnitClass (name)
			table.insert (playertable, 1, {death_table, last_cooldown, time_of_death, max_health, bossname, time, class})
			DeathGraphs:RefreshPlayerBox()
			
			--> survival stuff
			local playersurvival = DeathGraphs.current_survival_table [name]
			if (not playersurvival) then
				print ("playersurvival not found: ", name)
				local amt = 0
				for a, b in pairs (DeathGraphs.current_survival_table) do 
					if (a:find (name) or name:find (a)) then
						print ("search>", a, b, name)
					end
					amt = amt+1
				end
				print ("search",amt, "indexes")
				return
			end
			
			playersurvival.alive = false
			playersurvival.time = playersurvival.time + (time - playersurvival.alivesince)

		end
		
	end
	
	--> user clicked on button, need open or close window
	function DeathGraphs:OpenWindow()
	
		if (DeathGraphs.open) then	
			return DeathGraphs:CloseWindow()
		else
			DeathGraphs.open = true
		end
		
		--> show
		DeathGraphsFrame:Show()
		DeathGraphs.DeathGraphsFrame2:Show()
		return true
	end
	
	function DeathGraphs:CloseWindow()
		DeathGraphsFrame:Hide()
		DeathGraphs.open = false
		return true
	end
	
	DeathGraphsFrame:SetScript ("OnHide", function()
		DeathGraphs.open = false
	end)
	
	
	-->  hide icon on toolbar
	function DeathGraphs:HideIcon()
		DeathGraphs.showing = false
		DeathGraphs:HideToolbarIcon (DeathGraphs.ToolbarButton)
	end
	
	--> show icon on toolbar
	function DeathGraphs:ShowIcon()
		DeathGraphs.showing = true
		--> [1] button to show [2] button animation: "star", "blink" or true (blink)
		DeathGraphs:ShowToolbarIcon (DeathGraphs.ToolbarButton, "star")
	end
	
	--> create the button to show on toolbar [1] function OnClick [2] texture [3] tooltip [4] width or 14 [5] height or 14 [6] frame name or nil
	DeathGraphs.ToolbarButton = _detalhes.ToolBar:NewPluginToolbarButton (DeathGraphs.OpenWindow, [[Interface\GossipFrame\TaxiGossipIcon]], Loc ["STRING_TOOLTIP"], 12, 12, "DEATH_GRAPHS_BUTTON")
	--> setpoint anchors mod if needed
	DeathGraphs.ToolbarButton.y = 0
	DeathGraphs.ToolbarButton.x = 0
	
	
	--> build frames
	DeathGraphsFrame:SetFrameStrata ("MEDIUM")
	DeathGraphsFrame:SetPoint ("center", UIParent, "center")	
	DeathGraphsFrame:SetWidth (818)
	DeathGraphsFrame:SetHeight (454)
	local DeathGraphsFrame2 = DeathGraphsFrame
	local DeathGraphsFrame = CreateFrame ("frame", nil, DeathGraphsFrame, "ButtonFrameTemplate")
	DeathGraphs.DeathGraphsFrame2 = DeathGraphsFrame
	
	DeathGraphsFrame.portrait:SetTexture ([[Interface\BattlefieldFrame\UI-Battlefield-Icon]])
	DeathGraphsFrame.TitleText:SetText ("Death Logs")
		
	DeathGraphsFrame:SetScript ("OnHide", function()
		DeathGraphs.open = false
		DeathGraphsFrame2:Hide()
	end)

	DeathGraphsFrame:SetFrameStrata ("MEDIUM")
	DeathGraphsFrame:SetPoint ("center", DeathGraphsFrame2, "center")
	
	DeathGraphsFrame:SetWidth (818)
	DeathGraphsFrame:SetHeight (454)
	DeathGraphsFrame:EnableMouse (true)
	DeathGraphsFrame:SetResizable (false)
	DeathGraphsFrame:SetMovable (true)
	
	DeathGraphsFrame:SetScript ("OnMouseDown", 
					function (self, botao)
						if (botao == "LeftButton") then
							self:StartMoving()
							self.isMoving = true
						end
					end)
					
	DeathGraphsFrame:SetScript ("OnMouseUp", 
					function (self)
						if (self.isMoving) then
							self:StopMovingOrSizing()
							self.isMoving = false
						end
					end)
	
	--DeathGraphsFrame:SetBackdrop ({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
	--DeathGraphsFrame:SetBackdrop ({bgFile = [[Interface\FrameGeneral\UI-Background-Marble]], tile = true, tileSize = 128,
	--	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 32,
	--	insets = {left = 5, right = 5, top = 5, bottom = 5}})
	--DeathGraphsFrame:SetBackdropColor (.3, .3, .3, .3)
	--DeathGraphsFrame:SetBackdropColor (.7, .7, .7, 1)
	
	--> log lines
	
	local log
	local loglines = {}
	local linhas = {}
	local gradeframes = {}
	local gradelinhas = {}
	local bolinhas = {}
	
	local GetSpellInfo = _detalhes.getspellinfo
	local flag_logmouseover = false
	
	local refresh_loglines = function (self)
		local logs = log
	
		FauxScrollFrame_Update (DeathGraphs.logbox, #logs, 10, 14)
		
		local offset = FauxScrollFrame_GetOffset (DeathGraphs.logbox)
		--print ("offset", offset)
		
		for bar_index = 1, 10 do 
			local data = logs [bar_index + offset]

			if (data) then
			
				local type = data [1] --heal or damage
				local spellid = data [2] --spellid
				local amount = data [3] --amount healed or damaged
				local time = data [4] --time
				local life = data [5] --health
				local source = data [6] --source
			
				local spellname, _, spellicon = GetSpellInfo (spellid)
				if (spellname) then
					local line = loglines [bar_index]
					
					local _, class = UnitClass (source)
					local color = "FFFF9999"
					if (class) then
						local r, g, b = unpack (DeathGraphs.class_colors [class])
						r, g, b = DeathGraphs:hex (r), DeathGraphs:hex (g), DeathGraphs:hex (b)
						color = "FF"..r..g..b
					end
					
					if (type) then
						line.text:SetText ("|cFFFF9999-" .. DeathGraphs:ToK (amount) .. "|r |cFFFFFF99" .. spellname .. "|r from |c" .. color .. source .. "|r")
					else
						line.text:SetText ("|cFF99FF99+" .. DeathGraphs:ToK (amount) .. "|r |cFFFFFF99" .. spellname .. "|r from |c" .. color .. source .. "|r")
					end
					line.icon:SetTexture (spellicon)
					line.frame.index = bar_index + offset
				else
					local line = loglines [bar_index]
					line.text:SetText ("--x--x--")
					line.icon:SetTexture (nil)
				end
			
			end
		end
	end
	
	local logbox = CreateFrame ("scrollframe", "Details_DeathGraphs_LogBox", DeathGraphsFrame, "FauxScrollFrameTemplate")
	logbox:SetScript ("OnVerticalScroll", function (self, offset) FauxScrollFrame_OnVerticalScroll (self, offset, 14, refresh_loglines) end)
	logbox:SetPoint ("topleft", DeathGraphsFrame, "topleft", 324, -275)
	logbox:SetSize (462, 150)
	logbox:SetBackdrop ({bgFile = "Interface\\AddOns\\Details\\images\\background", tile = true, tileSize = 16,
	insets = {left = 0, right = 0, top = 0, bottom = 0}})
	logbox:SetBackdropColor (.1, .1, .1, .2)
	DeathGraphs.logbox = logbox
	
	local logarrow = logbox:CreateTexture (nil, "overlay")
	logarrow:SetTexture ([[Interface\CHATFRAME\ChatFrameExpandArrow]])
	logarrow:Hide()
	
	local lastline
	local line_on_enter = function (self)
	
		if (lastline) then
			lastline:GetScript ("OnLeave")(lastline)
		end
	
		if (self.index) then
			local line = gradeframes [math.abs (self.index-17)]
			flag_logmouseover = true
			line:GetScript ("OnEnter")(line)
			lastline = line
			
			self:SetBackdropColor (1, 1, 1, .2)
			logarrow:Show()
			logarrow:SetPoint ("right", self, "left", 0, 0)
		end
	end
	
	local line_on_leave = function (self)
		if (self.index) then
			local line = gradeframes [math.abs (self.index-17)]
			line:GetScript ("OnLeave")(line)
			self:SetBackdropColor (0, 0, 0, 0)
			logarrow:Hide()
			
			if (lastline) then
				lastline:GetScript ("OnLeave")(lastline)
			end
			lastline = nil
		end
	end
	
	--> log lines
	for i = 0, 9 do 
	
		local frame = CreateFrame ("frame", nil, logbox)
		frame:SetSize (462, 14)
		frame:SetPoint ("topleft", DeathGraphsFrame, "topleft", 342, -275 + (i*14*-1))
		frame:SetScript ("OnEnter", line_on_enter)
		frame:SetScript ("OnLeave", line_on_leave)
	
		frame:SetBackdrop ({bgFile = "Interface\\AddOns\\Details\\images\\background", tile = true, tileSize = 16,
		insets = {left = 0, right = 0, top = 0, bottom = 0}})
		frame:SetBackdropColor (0, 0, 0, 0)
	
		local icon = frame:CreateTexture (nil, "overlay")
		icon:SetPoint ("left", frame, "left")
		icon:SetSize (14, 14)
		local line = frame:CreateFontString (nil, "overlay", "GameFontNormal")
		line:SetPoint ("left", icon, "right", 2, 0)
		DeathGraphs:SetFontSize (line, 10)
		DeathGraphs:SetFontColor (line, "white")

		frame.id = i+1
		
		loglines [i+1] = {icon = icon, text = line, frame = frame}

	end
	

		
		local extra_desc = CreateFrame ("frame", nil, DeathGraphsFrame)
		extra_desc:SetFrameLevel (DeathGraphsFrame:GetFrameLevel()+2)
		extra_desc:SetSize (462, 20)
		extra_desc:SetPoint ("topleft", DeathGraphsFrame, "topleft", 324, -63)
		
		extra_desc.texture = extra_desc:CreateTexture (nil, "artwork")
		extra_desc.texture:SetTexture ([[Interface\ARCHEOLOGY\Arch-Progress-Bg]])
		extra_desc.texture:SetPoint ("topleft", extra_desc, "topleft", -12, 2)
		extra_desc.texture:SetSize (598, 36)
		
		local graphic_text = extra_desc:CreateFontString (nil, "overlay", "GameFontNormal")
		graphic_text:SetPoint ("left", extra_desc, "left", 2, 0)
		graphic_text:SetText ("Graphic")
	
	--> box graphic
		--local g = LibStub:GetLibrary ("LibGraph-2.0"):CreateGraphLine ("Details_DeathGraphs_Graph", DeathGraphsFrame, "topleft","topleft", 324, -20, 475, 375)
		local gframe = CreateFrame ("frame", nil, DeathGraphsFrame)
		gframe:SetSize (475, 160)
		gframe:SetPoint ("topleft", DeathGraphsFrame, "topleft", 324, -85)
		gframe.CustomLine = [[Interface\AddOns\Details\Libs\LibGraph-2.0\line]]
		
		local g = LibStub:GetLibrary ("LibGraph-2.0")
		
		--> graphic lines on enter and leave
		local on_enter = function (self)
			self:SetBackdropColor (.1, .1, .1, 0)
			local ball = bolinhas [self.id]
			ball:SetBlendMode ("ADD")
			local line = ball.line
			line:SetBlendMode ("ADD")
			
			if (not flag_logmouseover) then
				local offset = 7 - self.id
				if (offset < 0) then
					offset = 0
				end
				FauxScrollFrame_SetOffset (DeathGraphs.logbox, offset)
				refresh_loglines (DeathGraphs.logbox)
			end
			
			flag_logmouseover = false
			
			local id = self.id
			local offset = FauxScrollFrame_GetOffset (DeathGraphs.logbox)
			
			if (id < 8) then
				logarrow:Show()
				logarrow:SetPoint ("right", loglines [10].frame, "left", 0, 0)
			else
				local line = 17 - self.id
				logarrow:Show()
				logarrow:SetPoint ("right", loglines [line].frame, "left", 0, 0)
			end
			
			--[[
			local t = self.data
			local type = t [1] --heal or damage
			local spellid = t [2] --spellid
			local amount = t [3] --amount healed or damaged
			local time = t [4] --time
			local life = t [5] --health
			local source = t [6] --source
			
			print (type, life)
			--]]
			
		end
		local on_leave = function (self)
			self:SetBackdropColor (.1, .1, .1, .4)
			local ball = bolinhas [self.id]
			ball:SetBlendMode ("BLEND")
			local line = ball.line
			line:SetBlendMode ("BLEND")
			
		end
		
		--> graphic lines
		for i = 1, 16 do
			linhas [i] = g:DrawLine (gframe, 400, 500, 600, 700, 20, {1, 1, 1, 1}, "artwork")
			
			local f = CreateFrame ("frame", nil, gframe)
			f:SetPoint ("left", gframe, "left", (i-1)*29, 0)
			f:SetSize (29, 160)
			gradeframes [i] = f
			f.id = i
			f:SetScript ("OnEnter", on_enter)
			f:SetScript ("OnLeave", on_leave)
			f:SetBackdrop ({bgFile = "Interface\\AddOns\\Details\\images\\background", tile = true, tileSize = 16,
			insets = {left = 0, right = 0, top = 0, bottom = 0}})
			f:SetBackdropColor (.1, .1, .1, .4)
			
			local t = gframe:CreateTexture (nil, "background")
			t:SetSize (1, 160)
			t:SetPoint ("left", gframe, "left", i*29, 0)
			t:SetTexture (1, 1, 1, .1)
			gradelinhas [i] = t
			
			local b = f:CreateTexture (nil, "overlay")
			b:SetTexture ([[Interface\COMMON\Indicator-Yellow]])
			b:SetSize (16, 16)
			bolinhas [i] = b
		end
		
		function gframe:ShowGrid()
			for _, frame in ipairs (gradeframes) do 
				frame:Show()
			end
			for _, line in ipairs (gradelinhas) do 
				line:Show()
			end
			for _, bola in ipairs (bolinhas) do 
				bola:Show()
			end
		end
		function gframe:HideGrid()
			for _, frame in ipairs (gradeframes) do 
				frame:Hide()
			end
			for _, line in ipairs (gradelinhas) do 
				line:Hide()
			end
			for _, bola in ipairs (bolinhas) do 
				bola:Hide()
			end
		end

		function gframe:Reset()
			for i = #gframe.GraphLib_Lines_Used, 1, -1 do
				local line = tremove (gframe.GraphLib_Lines_Used)
				tinsert (gframe.GraphLib_Lines, line)
				line:Hide()
			end
		end
		
		--> timeline
		local timeline_bg = CreateFrame ("frame", nil, DeathGraphsFrame)
		timeline_bg:SetPoint ("topleft", gframe, "bottomleft", 0, -1)
		timeline_bg:SetPoint ("topright", gframe, "bottomright", 0, -1)
		timeline_bg:SetHeight (20)
		timeline_bg:SetBackdrop ({bgFile = "Interface\\AddOns\\Details\\images\\background", tile = true, tileSize = 16,
			insets = {left = 0, right = 0, top = 0, bottom = 0}})
		timeline_bg:SetBackdropColor (.1, .1, .1, .3)
		timeline_bg.labels = {}
		for i = 1, 16 do 
			local l = timeline_bg:CreateFontString (nil, "overlay", "GameFontNormal")
			DeathGraphs:SetFontSize (l, 9)
			DeathGraphs:SetFontColor (l, "silver")
			l:SetPoint ("left", timeline_bg, "left", (i-1) * 29, 0)
			timeline_bg.labels [i] = l
		end

		local red = {.9, .1, .1, 1}
		local green = {.1, .9, .1, 1}
		
		function DeathGraphs:ShowGraphicForDeath (data)
			
			DeathGraphs:ShowGraphicPanel()
			
			gframe:Reset()
			gframe:ShowGrid()
		
			local timeline = data [1]
			local max_health = data[4]

			--> working in progress
			table.sort (timeline, function (t1, t2)
			
				if (t2 [4] > t1 [4]) then --tempo maior vai pro final
					return false
					
				elseif (t2 [4] == t1 [4]) then
				
					if (not t2 [1] and not t2 [1]) then --ambos sao healing e no mesmo tempo 
						return t1 [5] > t2 [5] --life < life
						
					elseif (t2 [1] and t2 [1]) then --ambos sao dano e no mesmo tempo
						return t1 [5] < t2 [5] --life > life
						
					else --healing vem pra frente
						return not t1 [1]
						
					end 
					
				else
					return true
				end

			end)
			
			if (#timeline < 16) then
				for i = #timeline+1, 16 do
					table.insert (timeline, i, {false, 0, 0, data[6], max_health, ""})
				end
			end
			
			log = timeline
			
			refresh_loglines (DeathGraphs.logbox)
			
			local h = gframe:GetHeight()/100

			local o = 1
			local lastlife = 156

			for i = 16, 1, -1 do
				local t = timeline [i]
				if (type (t) == "table") then
					local type = t [1] --heal or damage
					local spellid = t [2] --spellid
					local amount = t [3] --amount healed or damaged
					local time = t [4] --time
					local life = t [5] --health
					local source = t [6] --source

					local plife = life / max_health * 100
					if (plife > 98) then
						plife = 98
					end
					plife = plife*h
					
					local line = g:DrawLine (gframe, (o-1)*29, lastlife, o*29, plife, 50, type and red or green, "overlay")
					bolinhas [o]:SetPoint ("bottomleft", gframe, "bottomleft", (o*29)-8, plife-8)
					bolinhas [o].line = line
					
					local clock = data[6] - time
					timeline_bg.labels [o]:SetText ("-" .. string.format ("%.1f", clock))
					
					local frame = gradeframes [o]
					frame.data = t
					
					lastlife = plife
					o = o + 1
				end
			end

		end
		
		gframe:Reset()
		gframe:HideGrid()


	
	--> box player
	
		local playerboxes = {}
		local segmentboxes = {}
		local playerselected = ""
		
		local refresh_function = function (self)
			local offset = FauxScrollFrame_GetOffset (self)

			function pairsByKeys (t, f)
				local a = {}
				for n in pairs(t) do table.insert(a, n) end
				table.sort(a, f)
				local i = 0      -- iterator variable
				local iter = function ()   -- iterator function
					i = i + 1
					if a[i] == nil then return nil
					else return a[i], t[a[i]]
					end
				end
				return iter
			end
			
			local t = {}
			local i = 1
			for name, tble in pairsByKeys (DeathGraphs.players) do
				t [i] = {name, tble}
				i = i+1
			end
			
			local list = t

			for bar_index = 1, 17 do 
				local data = list [bar_index + offset]

				if (data) then
				
					local playername = data[1]
					if (playername:find ("-")) then
						playername = playername:gsub (("-.*"), "")
					end
				
					playerboxes [bar_index].text:SetText (playername)
					
					local _, class = UnitClass (data[1])

					if (not class) then
						if (data[2][1][7]) then
							class = data[2][1][7]
						end
					end
					
					if (playerselected:find (playername)) then
						playerboxes [bar_index]:SetBackdropColor (1, 1, 1, .8)
						playerboxes [bar_index].is_selected = true
					else
						playerboxes [bar_index]:SetBackdropColor (.1, .1, .1, .2)
						playerboxes [bar_index].is_selected = false
					end
					
					playerboxes [bar_index].playername = playername
					
					if (class) then
						playerboxes [bar_index].text:SetTextColor (unpack (DeathGraphs.class_colors [class]))
						playerboxes [bar_index].icon:SetTexture (DeathGraphs.class_icons_small)
						playerboxes [bar_index].icon:SetTexCoord (unpack (DeathGraphs.class_coords [class]))
					else
						playerboxes [bar_index].text:SetTextColor (1, .8, 0, 1)
						playerboxes [bar_index].icon:SetTexture (nil)
					end
					
					playerboxes [bar_index].t = data
					playerboxes [bar_index]:Show()
				else
					playerboxes [bar_index]:Hide()
					playerboxes [bar_index].t = nil
				end
			end
		
		end

		local playerbox = CreateFrame ("scrollframe", "Details_DeathGraphs_PlayerBox", DeathGraphsFrame, "FauxScrollFrameTemplate")
		playerbox:SetScript ("OnVerticalScroll", function (self, offset) FauxScrollFrame_OnVerticalScroll (self, offset, 20, refresh_function) end)
		playerbox:SetPoint ("topleft", DeathGraphsFrame, "topleft", 10, -85)
		playerbox:SetSize (135, 338)
		playerbox:SetBackdrop ({bgFile = "Interface\\AddOns\\Details\\images\\background", tile = true, tileSize = 16,
		insets = {left = 0, right = 0, top = 0, bottom = 0}})
		playerbox:SetBackdropColor (.1, .1, .1, .2)
		
		--title
		local extra_desc = CreateFrame ("frame", nil, DeathGraphsFrame)
		extra_desc:SetFrameLevel (DeathGraphsFrame:GetFrameLevel()+2)
		extra_desc:SetSize (135, 20)
		extra_desc:SetPoint ("topleft", DeathGraphsFrame, "topleft", 10, -63)
		
		extra_desc.texture = extra_desc:CreateTexture (nil, "artwork")
		extra_desc.texture:SetTexture ([[Interface\ARCHEOLOGY\Arch-Progress-Bg]])
		extra_desc.texture:SetPoint ("topleft", extra_desc, "topleft", -4, 2)
		extra_desc.texture:SetSize (193, 36)
		
		extra_desc.text1 = extra_desc:CreateFontString (nil, "overlay", "GameFontNormal")
		extra_desc.text1:SetPoint ("left", extra_desc, "left", 2, 0)
		extra_desc.text1:SetText ("Player Name")
		
		--scroll background
		local upper = DeathGraphsFrame:CreateTexture (nil, "artwork")
		upper:SetTexture ([[Interface\ClassTrainerFrame\UI-ClassTrainer-ScrollBar]])
		upper:SetTexCoord (0, 0.5, 0, 1)
		upper:SetSize (32, 128)
		local down = DeathGraphsFrame:CreateTexture (nil, "artwork")
		down:SetTexture ([[Interface\ClassTrainerFrame\UI-ClassTrainer-ScrollBar]])
		down:SetTexCoord (0.5, 1, 0, 1)
		down:SetSize (32, 128)
		local middle = DeathGraphsFrame:CreateTexture (nil, "artwork")
		middle:SetTexture ([[Interface\ClassTrainerFrame\UI-ClassTrainer-ScrollBar]])
		middle:SetTexCoord (0, 0.5, 0.203125, 1)
		middle:SetSize (32, 128)
		
		upper:SetPoint ("topright", playerbox, "topright", 29, 4)
		down:SetPoint ("bottomright", playerbox, "bottomright", 27, -2)
		middle:SetPoint ("top", upper, "bottom", 0, 5)
		middle:SetPoint ("bottom", down, "top", 0, -10)
		
		
		local player_on_enter = function (self)
			if (not self.is_selected) then
				self:SetBackdropColor (1, 1, 1, .3)
			end
		end
		local player_on_leave = function (self)
			if (not self.is_selected) then
				self:SetBackdropColor (.1, .1, .1, .2)
			end
		end
		
		local player_selected = DeathGraphsFrame:CreateFontString (nil, "overlay", "GameFontNormal")
		player_selected:SetPoint ("topleft", DeathGraphsFrame, "topleft", 70, -37)
		--player_selected:
		
		for i = 0, 16 do 
			local button = CreateFrame ("button", nil, DeathGraphsFrame)
			button:SetPoint ("topleft", playerbox, "topleft", 0, i*20*-1)
			button:SetSize (135, 18)
			button:SetBackdrop ({bgFile = "Interface\\AddOns\\Details\\images\\background", tile = true, tileSize = 16,
			insets = {left = 0, right = 0, top = 0, bottom = 0}})
			button:SetBackdropColor (.1, .1, .1, .2)
			playerboxes [#playerboxes+1] = button
			
			button:SetScript ("OnEnter", player_on_enter)
			button:SetScript ("OnLeave", player_on_leave)
			
			local icon = button:CreateTexture (nil, "Overlay")
			icon:SetPoint ("left", button, "left", 2, 0)
			icon:SetSize (17, 17)
			local text = button:CreateFontString (nil, "overlay", "GameFontNormal")
			text:SetPoint ("left", icon, "right", 2, 0)
			
			button.icon = icon
			button.text = text
			
			button:SetScript ("OnClick", function()
				if (button.t) then
					local playername = button.t [1]
					playerselected = playername
					DeathGraphs:ShowSegments (playername)
					
					for i = 1, 17 do 
						playerboxes [i].is_selected = false
						playerboxes [i]:SetBackdropColor (.1, .1, .1, .2)
					end
					
					button:SetBackdropColor (1, 1, 1, .8)
					button.is_selected = true
				end
			end)
		end
		
		local amt = 0
		for _, _ in pairs (DeathGraphs.players) do amt = amt+1 end
		FauxScrollFrame_Update (playerbox, amt, 18, 20)
		refresh_function (playerbox)
		
		function DeathGraphs:RefreshPlayerBox()
			FauxScrollFrame_Update (playerbox, amt, 18, 20)
			refresh_function (playerbox)
		end
	
	--> box segment

		local segmentselected = ""
		
		local segmentbox = CreateFrame ("scrollframe", "Details_DeathGraphs_SegmentBox", DeathGraphsFrame, "FauxScrollFrameTemplate")
		segmentbox:SetScript ("OnVerticalScroll", function (self, offset) FauxScrollFrame_OnVerticalScroll (self, offset, 20, refresh_function2) end)
		segmentbox:SetPoint ("topleft", DeathGraphsFrame, "topleft", 172, -85)
		segmentbox:SetSize (125, 338)
		segmentbox:SetBackdrop ({bgFile = "Interface\\AddOns\\Details\\images\\background", tile = true, tileSize = 16,
		insets = {left = 0, right = 0, top = 0, bottom = 0}})
		segmentbox:SetBackdropColor (.1, .1, .1, .2)
		
		local extra_desc = CreateFrame ("frame", nil, DeathGraphsFrame)
		extra_desc:SetFrameLevel (DeathGraphsFrame:GetFrameLevel()+2)
		extra_desc:SetSize (135, 20)
		extra_desc:SetPoint ("topleft", DeathGraphsFrame, "topleft", 172, -63)
		
		extra_desc.texture = extra_desc:CreateTexture (nil, "artwork")
		extra_desc.texture:SetTexture ([[Interface\ARCHEOLOGY\Arch-Progress-Bg]])
		extra_desc.texture:SetPoint ("topleft", extra_desc, "topleft", -4, 2)
		extra_desc.texture:SetSize (187, 36)
		
		extra_desc.text1 = extra_desc:CreateFontString (nil, "overlay", "GameFontNormal")
		extra_desc.text1:SetPoint ("left", extra_desc, "left", 2, 0)
		extra_desc.text1:SetText ("Segment")		
		
		--scroll background
		local upper = DeathGraphsFrame:CreateTexture (nil, "artwork")
		upper:SetTexture ([[Interface\ClassTrainerFrame\UI-ClassTrainer-ScrollBar]])
		upper:SetTexCoord (0, 0.5, 0, 1)
		upper:SetSize (32, 128)
		local down = DeathGraphsFrame:CreateTexture (nil, "artwork")
		down:SetTexture ([[Interface\ClassTrainerFrame\UI-ClassTrainer-ScrollBar]])
		down:SetTexCoord (0.5, 1, 0, 1)
		down:SetSize (32, 128)
		local middle = DeathGraphsFrame:CreateTexture (nil, "artwork")
		middle:SetTexture ([[Interface\ClassTrainerFrame\UI-ClassTrainer-ScrollBar]])
		middle:SetTexCoord (0, 0.5, 0.203125, 1)
		middle:SetSize (32, 128)

		upper:SetPoint ("topright", segmentbox, "topright", 29, 4)
		down:SetPoint ("bottomright", segmentbox, "bottomright", 27, -2)
		middle:SetPoint ("top", upper, "bottom", 0, 5)
		middle:SetPoint ("bottom", down, "top", 0, -10)
		
		local segment_on_enter = function (self)
			if (not self.is_selected) then
				self:SetBackdropColor (1, 1, 1, .3)
			end
		end
		local segment_on_leave = function (self)
			if (not self.is_selected) then
				self:SetBackdropColor (.1, .1, .1, .2)
			end
		end
		
		for i = 0, 16 do 
			local button = CreateFrame ("button", nil, DeathGraphsFrame)
			button:SetPoint ("topleft", segmentbox, "topleft", 0, i*20*-1)
			button:SetSize (125, 18)
			button:SetBackdrop ({bgFile = "Interface\\AddOns\\Details\\images\\background", tile = true, tileSize = 16,	insets = {left = 0, right = 0, top = 0, bottom = 0}})
			button:SetBackdropColor (.1, .1, .1, .2)
			
			button:SetScript ("OnEnter", segment_on_enter)
			button:SetScript ("OnLeave", segment_on_leave)
			
			local text = button:CreateFontString (nil, "overlay", "GameFontNormal")
			text:SetPoint ("left", button, "left", 2, 0)
			DeathGraphs:SetFontSize (text, 10)
			DeathGraphs:SetFontColor (text, "white")
			text:SetWidth (125)
			text:SetHeight (18)
			text:SetJustifyH ("left")
			button.icon = icon
			button.text = text
			
			segmentboxes [#segmentboxes+1] = button
			
			button:SetScript ("OnClick", function()
				if (button.t) then
					local data = button.t
					DeathGraphs:ShowGraphicForDeath (data)
					
					player_selected:SetText (playerselected .. " -> " .. button.text:GetText())
					segmentselected = button.text:GetText()

					for i = 1, 17 do 
						segmentboxes [i].is_selected = false
						segmentboxes [i]:SetBackdropColor (.1, .1, .1, .2)
					end
					
					button:SetBackdropColor (1, 1, 1, .8)
					button.is_selected = true
					
				end
			end)
		end
	
		function DeathGraphs:ShowSegments (playername)
		
			local segments = DeathGraphs.players [playername]
		
			FauxScrollFrame_Update (segmentbox, math.max (#segments, 18), 17, 20)
			
			local offset = FauxScrollFrame_GetOffset (segmentbox)
			
			for bar_index = 1, 17 do 
				local data = segments [bar_index + offset]

				if (data) then
				
					local minutos, segundos = math.floor (data[3]/60), math.floor (data[3]%60)
					local str = minutos .. "m " .. segundos .. "s" .. " - " .. data[5]
					segmentboxes [bar_index].text:SetText (str)
					segmentboxes [bar_index].t = data
					segmentboxes [bar_index]:Show()
					
					if (segmentselected:find (str)) then
						segmentboxes [bar_index]:SetBackdropColor (1, 1, 1, .8)
						segmentboxes [bar_index].is_selected = true
					else
						segmentboxes [bar_index]:SetBackdropColor (.1, .1, .1, .2)
						segmentboxes [bar_index].is_selected = false
					end
					
				else
					segmentboxes [bar_index]:Hide()
					segmentboxes [bar_index].t = nil
				end
			end
			
			segmentboxes [1]:Click()

		end
		
		FauxScrollFrame_Update (segmentbox, 18, 17, 20)
	
	
	--> extra panels
	
	local extra_painel_showing
	local extra_panel_lines = {}
	local extra_panel_data
	
	function DeathGraphs:HideGraphicPanel()
		logbox:Hide()
		gframe:Hide()
		timeline_bg:Hide()
		graphic_text:Hide()
		DeathGraphs.extra_desc:Show()
		DeathGraphs.extra_painel:Show()

	end
	
	function DeathGraphs:ShowGraphicPanel()
		logbox:Show()
		gframe:Show()
		timeline_bg:Show()
		graphic_text:Show()
		
		for _, row in ipairs (extra_panel_lines) do
			row:Hide()
		end
		
		DeathGraphs.extra_desc:Hide()
		DeathGraphs.extra_painel:Hide()

	end
	
	local refresh_extra_panel = function (self)
		local player_table = extra_panel_data
		
		local offset = FauxScrollFrame_GetOffset (DeathGraphs.extra_painel)
		
		if (extra_painel_showing == 1) then
			for bar_index = 1, 17 do
				local data = player_table [bar_index + offset]

				if (data) then
					extra_panel_lines [bar_index].text:SetText (data [1])
					extra_panel_lines [bar_index].text2:SetText (data [2].time)
					extra_panel_lines [bar_index].tooltip = nil
					extra_panel_lines [bar_index]:Show()
				else
					extra_panel_lines [bar_index]:Hide()
				end
			end

		elseif (extra_painel_showing == 2) then
			for bar_index = 1, 17 do
				local data = player_table [bar_index + offset]

				if (data) then
					extra_panel_lines [bar_index].text:SetText (data [1])
					extra_panel_lines [bar_index].text2:SetText (data [2])
					extra_panel_lines [bar_index].tooltip = {}
					for _, death in ipairs (data [3]) do 
						local minutos, segundos = math.floor (death.time/60), math.floor (death.time%60)
						tinsert (extra_panel_lines [bar_index].tooltip, minutos .. "m " .. segundos .. "s" .. " on " .. death.boss )
					end
					extra_panel_lines [bar_index]:Show()
				else
					extra_panel_lines [bar_index]:Hide()
				end
			end
		end
	end

	local extra_painel = CreateFrame ("scrollframe", "Details_DeathGraphs_ExtraPanel", DeathGraphsFrame, "FauxScrollFrameTemplate")
	extra_painel:SetScript ("OnVerticalScroll", function (self, offset) FauxScrollFrame_OnVerticalScroll (self, offset, 20, refresh_extra_panel) end)
	extra_painel:SetPoint ("topleft", DeathGraphsFrame, "topleft", 324, -85)
	extra_painel:SetSize (462, 340)
	extra_painel:SetBackdrop ({bgFile = "Interface\\AddOns\\Details\\images\\background", tile = true, tileSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}})
	extra_painel:SetBackdropColor (.1, .1, .1, .2)
	DeathGraphs.extra_painel = extra_painel

	local extra_desc = CreateFrame ("frame", nil, DeathGraphsFrame)
	extra_desc:SetFrameLevel (DeathGraphsFrame:GetFrameLevel()+2)
	extra_desc:SetSize (462, 20)
	extra_desc:SetPoint ("topleft", DeathGraphsFrame, "topleft", 324, -63)
	
	extra_desc.texture = extra_desc:CreateTexture (nil, "artwork")
	extra_desc.texture:SetTexture ([[Interface\ARCHEOLOGY\Arch-Progress-Bg]])
	extra_desc.texture:SetPoint ("topleft", extra_desc, "topleft", -12, 2)
	extra_desc.texture:SetSize (598, 36)
	
	extra_desc.text1 = extra_desc:CreateFontString (nil, "overlay", "GameFontNormal")
	extra_desc.text2 = extra_desc:CreateFontString (nil, "overlay", "GameFontNormal")
	extra_desc.text1:SetPoint ("left", extra_desc, "left", 2, 0)
	extra_desc.text2:SetPoint ("right", extra_desc, "right", -2, 0)
	extra_desc.text1:SetText ("left")
	extra_desc.text2:SetText ("right")
	DeathGraphs.extra_desc = extra_desc
	
	local extra_on_enter = function (self)
		self:SetBackdropColor (1, 1, 1, .3)
		if (self.tooltip) then
			GameCooltip:Reset()
			GameCooltip:SetOwner (self)
			for _, line in ipairs (self.tooltip) do
				GameCooltip:AddLine (line)
			end
			GameCooltip:Show()
		end
	end
	local extra_on_leave = function (self)
		self:SetBackdropColor (.1, .1, .1, .2)
		if (self.tooltip) then
			GameCooltip:Close()
		end
	end
	
	for i = 0, 16 do 
		local row = CreateFrame ("frame", nil, DeathGraphsFrame)
		row:SetPoint ("topleft", extra_painel, "topleft", 0, i*20*-1)
		row:SetSize (462, 18)
		row:SetBackdrop ({bgFile = "Interface\\AddOns\\Details\\images\\background", tile = true, tileSize = 16,	insets = {left = 0, right = 0, top = 0, bottom = 0}})
		row:SetBackdropColor (.1, .1, .1, .2)
		
		row:SetScript ("OnEnter", extra_on_enter)
		row:SetScript ("OnLeave", extra_on_leave)
		
		local text = row:CreateFontString (nil, "overlay", "GameFontNormal")
		text:SetPoint ("left", row, "left", 2, 0)
		DeathGraphs:SetFontSize (text, 10)
		DeathGraphs:SetFontColor (text, "white")
		text:SetJustifyH ("left")
		
		local text2 = row:CreateFontString (nil, "overlay", "GameFontNormal")
		text2:SetPoint ("right", row, "right", -2, 0)
		DeathGraphs:SetFontSize (text2, 10)
		DeathGraphs:SetFontColor (text2, "white")
		text2:SetJustifyH ("right")
		
		row.icon = icon
		row.text = text
		row.text2 = text2
		
		extra_panel_lines [#extra_panel_lines+1] = row
	end


--> survival button
	local survivalbutton = CreateFrame ("button", "Details_DeathGraphs_SurvivalButton", DeathGraphsFrame, "OptionsButtonTemplate")
	survivalbutton:SetPoint ("topright", DeathGraphsFrame, "topright", -157, -35)
	survivalbutton:SetSize (150, 25)
	survivalbutton:SetText (Loc ["STRING_SURVIVAL"])
	
	survivalbutton:SetScript ("OnClick", function (self)
		--> hide main panels
		DeathGraphs:HideGraphicPanel()
		extra_painel_showing = 1
		
		extra_desc.text1:SetText ("Player Name")
		extra_desc.text2:SetText ("Survival Score")
		
		local sorted = {}
		for name, tble in pairs (DeathGraphs.survival) do 
			sorted [#sorted+1] = {name, tble}
		end
		
		table.sort (sorted, function (t1, t2) return t1[2].time > t2[2].time end)
		
		FauxScrollFrame_Update (extra_painel, #sorted, 17, 20)
		extra_panel_data = sorted
		refresh_extra_panel (extra_painel)
	end)
		
	
	--> battleress button
		local battleressbutton = CreateFrame ("button", "Details_DeathGraphs_BattleressButton", DeathGraphsFrame, "OptionsButtonTemplate")
		battleressbutton:SetPoint ("topright", DeathGraphsFrame, "topright", -7, -35)
		battleressbutton:SetSize (150, 25)
		battleressbutton:SetText (Loc ["STRING_BRESS"])
		
		battleressbutton:SetScript ("OnClick", function (self)
			--> hide main panels
			DeathGraphs:HideGraphicPanel()
			extra_painel_showing = 2
			
			extra_desc.text1:SetText ("Player Name")
			extra_desc.text2:SetText ("Amount")
			
			local sorted = {}
			for name, ress_table in pairs (DeathGraphs.battleress) do
				sorted [#sorted+1] = {name, #ress_table, ress_table}
			end
			
			table.sort (sorted, function (t1, t2) return t1[2] > t2[2] end)
			
			FauxScrollFrame_Update (extra_painel, #sorted, 17, 20)
			extra_panel_data = sorted
			refresh_extra_panel (extra_painel)
		end)
		
		
	DeathGraphs:ShowGraphicPanel()
	
	if (debugmode) then
		DeathGraphs:ShowIcon()
		
	end
end

function DeathGraphs:OnBattleRess (token, _time, who_serial, who_name, who_flags, alvo_serial, alvo_name, alvo_flags, spellid, spellname)
	
	if (DeathGraphs.in_combat) then
		
		local combat = DeathGraphs:GetCombat ("current")
		
		if (combat.is_boss or debugmode) then
			--> survival stuff
			local playersurvival = DeathGraphs.current_survival_table [alvo_name]
			if (not playersurvival) then
				print ("payer not found on BRES", alvo_name)
				return
			end
			playersurvival.alive = true
			playersurvival.alivesince = _time
			
			--> battle ress
			local playerbattleress = DeathGraphs:GetPlayerTable (alvo_name, "battleress")
			local bossname = combat.is_boss and combat.is_boss.name or ""
			table.insert (playerbattleress, 1, {boss = bossname, combat = DeathGraphs.combat_id, time = combat:GetCombatTime()})
		end
	end
	
end

function DeathGraphs:OnDeath (token, time, who_serial, who_name, who_flags, alvo_serial, alvo_name, alvo_flags, death_table, last_cooldown, time_of_death, max_health)

	local combat = DeathGraphs:GetCombat ("current")
	
	DeathGraphs:AddDeath (alvo_serial, alvo_name, death_table, last_cooldown, time_of_death, max_health, time)
	
end

function DeathGraphs:OnEvent (_, event, ...)

	if (event == "ADDON_LOADED") then
		local AddonName = select (1, ...)
		if (AddonName == "Details_DeathGraphs") then
			
			if (_G._detalhes) then
				
				--> catch data
				DeathGraphs.players = _detalhes_databaseDeathGraphsP or {}
				DeathGraphs.survival = _detalhes_databaseDeathGraphsS or {}
				DeathGraphs.battleress = _detalhes_databaseDeathGraphsB or {}
				_detalhes_databaseDeathGraphsP = DeathGraphs.players
				_detalhes_databaseDeathGraphsS = DeathGraphs.survival
				_detalhes_databaseDeathGraphsB = DeathGraphs.battleress
				
				--> create widgets
				CreatePluginFrames()

				--> core version required
				local MINIMAL_DETAILS_VERSION_REQUIRED = 14
				
				--> Install
				local install = _G._detalhes:InstallPlugin ("TOOLBAR", Loc ["STRING_PLUGIN_NAME"], "placeholder string", DeathGraphs, "DETAILS_PLUGIN_DEATH_GRAPHICS", MINIMAL_DETAILS_VERSION_REQUIRED)
				if (type (install) == "table" and install.error) then
					print (install.error)
				end
				
				--> Install hook
				_G._detalhes:InstallHook (DETAILS_HOOK_DEATH, DeathGraphs.OnDeath)
				_G._detalhes:InstallHook (DETAILS_HOOK_BATTLERESS, DeathGraphs.OnBattleRess)
				
				--> Register needed events
				_G._detalhes:RegisterEvent (DeathGraphs, "COMBAT_PLAYER_ENTER")
				_G._detalhes:RegisterEvent (DeathGraphs, "COMBAT_PLAYER_LEAVE")
				_G._detalhes:RegisterEvent (DeathGraphs, "DETAILS_DATA_RESET")

				--DeathGraphs:OpenWindow()
				
			end
		end
	end
end
